
# convert method implementation class

from .operators import *

class Convertor:

    def __init__(self):
        pass

    def coco2csv(self):
        # TODO: convert coco to csv
        pass

    def coco2pascalvoc(self):
        # TODO: convert coco to pascal VOC
        pass

    def csv2coco(self):
        # TODO: convert .csv into coco
        pass

    def csv2pascalvoc(self):
        # TODO: convert .csv into pascal VOC
        pass

    def pascalvoc2coco(self):
        # TODO: convert pascal VOC into coco
        pass

    def pascalvoc2csv(self):
        # TODO: convert pascal VOC into .csv
        pass

